$NOMOD51
$INCLUDE (reg51.inc)

; ***** BEGIN LICENSE BLOCK *****
; Version: MPL 1.1
;
; The contents of this file are subject to the Mozilla Public License Version 
; 1.1 (the "License"); you may not use this file except in compliance with 
; the License. You may obtain a copy of the License at 
; http://www.mozilla.org/MPL/
;
; Software distributed under the License is distributed on an "AS IS" basis,
; WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
; for the specific language governing rights and limitations under the
; License.
;
; The Original Code is firmware for the MiniDMX12 device.
;
; The Initial Developer of the Original Code is
; Tomsoft.
; Portions created by Tomsoft are Copyright (C) 2009
; Tomsoft. All Rights Reserved.
;
; Contributor(s):
;   Tommy Sools <tom@sools.com>
;
; ***** END LICENSE BLOCK *****

; MiniDMX12 V1.0 for AT89C4051

; V1.0:
; initial version

; 16 MHz quartz = 16000000 Hz
; 1 pulse = 0,0000000625 seconds
; 1 cycle = 0,0000000625 seconds * 12 pulses = 0,00000075 seconds = 0,00075 milliseconds
; 16 bit timer is being used, meaning every 65536 cycles it will overflow.
; 65536 cycles take 65536 * 0,00075 milliseconds = 49,152 milliseconds.
; The DMX signal can only be considered to be lost after 1 second = 1000 milliseconds =>
; only after 1000 milliseconds / 49,152 milliseconds = 20,345052083333333333333333333333 timer overflows ~ 21 timer overflows.

; Slot 1 up to 12: power settings

;   0 - 127 power off
; 128 - 255 power on

		NAME		uzuipt

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

switch1			EQU	P1.0
switch2			EQU	P1.1
switch3			EQU	P1.2
switch4			EQU	P1.3
switch5			EQU	P1.4
switch6			EQU	P1.5
switch7			EQU	P1.6
switch8			EQU	P1.7
switch9			EQU	P3.5
switch10		EQU	P3.4
switch11		EQU	P3.3
switch12		EQU	P3.2

_jumper1		EQU	P3.7 ; "dipswitch 9" for a value of 256
_jumper0		EQU	P3.1 ; "dipswitch 8" for a value of 128

; jumper1 not present & jumper0 not present = service mode
; jumper1 not present & jumper0 present = address 128
; jumper2 present & jumper0 not present = address 256
; jumper2 present & jumper0 present = address 384

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

dmxpointer		EQU	R0   		; DMX value pointer
dmxcounter		EQU	R1   		; local DMX table counter
signallostcountdown	EQU	R2
tempaddressh		EQU	R5
tempaddressl		EQU	R6
loopcountdown		EQU	R7

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

tablesize		EQU	14	; size of DMX table (break + start code + 12 DMX values)
signallostcountdowninit	EQU	21	; number of times the timer must overflow before the signal will be considered to be lost

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

?STACK		SEGMENT	IDATA		; ?STACK goes into IDATA RAM.
		RSEG	?STACK		; switch to ?STACK segment.
		DS	20

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

?BIT		SEGMENT	BIT		; segment for BIT RAM.
	   	RSEG	?BIT		; switch to ?BIT segment
dmxsignalcheck: 	DBIT	1	; DMX signal received bit
dmxsignal: 		DBIT	1	; DMX signal present bit

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

?DATA		SEGMENT	DATA		; segment for DATA RAM.
	   	RSEG	?DATA		; switch to this data segment
initaddressh:		DS	1		; initial processed high address byte
initaddressl:		DS	1		; initial processed low address byte
delay1:			DS	1
delay2:			DS	1
delay3:			DS	1
table:   		DS	tablesize	; reserve bytes for DMX table
addressh:		DS	1		; address high byte
addressl:		DS	1		; address low byte

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

%*DEFINE (ADDRESS_REFRESH) LOCAL addressh_enabled, addressl_enabled, address_not_zero, address_end (
		MOV	tempaddressh,#1
		JNB	_jumper1,%addressh_enabled	; retrieve high address byte
		MOV	tempaddressh,#0
%addressh_enabled:
		MOV	tempaddressl,#128
		JNB	_jumper0,%addressl_enabled	; retrieve low address byte
		MOV	tempaddressl,#0
%addressl_enabled:
		JNB	_jumper0,%address_not_zero	; check if low address byte is zero
		JNB	_jumper1,%address_not_zero	; check if high address byte is zero
		; the full address is zero, so use address 1
		SJMP	%address_end

%address_not_zero:	; the full address is not zero
; Decrement low address byte by 1, so that the address set using the dipswitches will be one-based rather than zero-based.
; This way a zero setting for the address could be used for special purposes.
; For address 1: initaddressh = 0; initaddressl = 0
; For address 2: initaddressh = 0; initaddressl = 1
; For address 3: initaddressh = 0; initaddressl = 2
; For address 128 (jumper 0 enabled): initaddressh = 0; initaddressl = 127
; For address 256 (jumper 1 enabled): initaddressh = 0; initaddressl = 255
; For address 384 (jumper 1 & 0 enabled): initaddressh = 1; initaddressl = 127

		MOV	A,tempaddressl
		CLR	C
		SUBB	A,#1
		MOV	tempaddressl,A
		MOV	A,tempaddressh
		SUBB	A,#0
		MOV	tempaddressh,A

%address_end:	MOV	initaddressh,tempaddressh
		MOV	initaddressl,tempaddressl
)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

		CSEG	AT  0			; absolute Segment at Address 0
		LJMP	start			; reset location (jump to start)

?CODE		SEGMENT	CODE
		RSEG	?CODE			; switch to this code segment

start:          MOV	SP,#?STACK-1   		; assign stack at beginning

init:		; disable all switches
		CLR	switch1
		CLR	switch2
		CLR	switch3
		CLR	switch4
		CLR	switch5
		CLR	switch6
		CLR	switch7
		CLR	switch8
		CLR	switch9
		CLR	switch10
		CLR	switch11
		CLR	switch12

		CLR	dmxsignal
		CLR	dmxsignalcheck

		MOV	signallostcountdown,#signallostcountdowninit

		; check if the full address is zero
		JNB	_jumper0,address_not_zero	; check if low address byte is zero
		JNB	_jumper1,address_not_zero	; check if high address byte is zero

		; address is zero, so go into service mode
		LJMP	service_mode

address_not_zero:
		%ADDRESS_REFRESH

		MOV	dmxpointer,#table+tablesize	; initialize DMX value pointer
emptytable:	DEC	dmxpointer
		MOV	@dmxpointer,#0
		CJNE	dmxpointer,#table,emptytable

		MOV     SCON,#10001000B		; mode 2 (9-bit UART, fixed baud rate),
						; no automatic address recognition, serial reception disabled,
						; set send bit 8, unset receive bit 8,
						; unset transmit and receive interrupt flags

		SETB	PS			; set high priority for serial port interrupt
		SETB    ES	       		; enable serial interrupt

		ORL	TMOD,#0000001B		; set timer 0 to 16-bit timer

		SETB	ET0	       		; enable timer 0 interrupt

		SETB    EA	      		; enable global interrupts
		SETB	REN			; enable serial reception
		SETB	TR0			; enable timer 0

		;CLR	_ledpower		; set power LED

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

%*DEFINE (UPDATESWITCH (address, switchbit)) LOCAL checkswitch, enableswitch, checkswitchend (
		; output switch status according to DMX value
		MOV	dmxcounter,#table+1+%address
		CJNE	@dmxcounter,#127,%checkswitch
%checkswitch:	JNC	%enableswitch	; if @dmxcounter is greater than or equal to #128, enable switch
		CLR	%switchbit
		SJMP	%checkswitchend
%enableswitch:	SETB	%switchbit
%checkswitchend:
)

loop:
		%ADDRESS_REFRESH

		; check if any DMX signal is present at all
		MOV	A,signallostcountdown
		JNZ	signalnotlost

signallost:	; disable all switches
		CLR	switch1
		CLR	switch2
		CLR	switch3
		CLR	switch4
		CLR	switch5
		CLR	switch6
		CLR	switch7
		CLR	switch8
		CLR	switch9
		CLR	switch10
		CLR	switch11
		CLR	switch12
		LJMP	loop

signalnotlost:
		%UPDATESWITCH(1, switch1)
		%UPDATESWITCH(2, switch2)
		%UPDATESWITCH(3, switch3)
		%UPDATESWITCH(4, switch4)
		%UPDATESWITCH(5, switch5)
		%UPDATESWITCH(6, switch6)
		%UPDATESWITCH(7, switch7)
		%UPDATESWITCH(8, switch8)
		%UPDATESWITCH(9, switch9)
		%UPDATESWITCH(10, switch10)
		%UPDATESWITCH(11, switch11)
		%UPDATESWITCH(12, switch12)
		LJMP	loop


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

		; approx. 500 millseconds delay ==> 666666,66666666666666666666666667 cycles are neccesary
		; total 716297 cycles ==> 537,22275 milliseconds
%*DEFINE (DELAY) LOCAL ldelay, ldelay2, ldelay3 (
		MOV	delay1,#255	; 2 cycles

%ldelay:	MOV	delay2,#255	; 2 cycles * 255 = 510

%ldelay2:	NOP			; 1 cycle * 255 * 255 = 65025
		NOP			; 1 cycle * 255 * 255 = 65025
		NOP			; 1 cycle * 255 * 255 = 65025
		NOP			; 1 cycle * 255 * 255 = 65025
		NOP			; 1 cycle * 255 * 255 = 65025
		NOP			; 1 cycle * 255 * 255 = 65025
		NOP			; 1 cycle * 255 * 255 = 65025
		NOP			; 1 cycle * 255 * 255 = 65025
		NOP			; 1 cycle * 255 * 255 = 65025
		DJNZ	delay2,%ldelay2	; 2 cycles * 255 * 255 = 130050
		DJNZ	delay1,%ldelay	; 2 cycles * 255 = 510
)

service_mode:
service_enableall_loop:
		; enable all switches.
		; switch to running light mode if jumper 0 is switched on.
		JNB	_jumper0,service_runninglight
		; enable all switches
		SETB	switch1
		SETB	switch2
		SETB	switch3
		SETB	switch4
		SETB	switch5
		SETB	switch6
		SETB	switch7
		SETB	switch8
		SETB	switch9
		SETB	switch10
		SETB	switch11
		SETB	switch12
		SJMP	service_enableall_loop

service_runninglight:
		; disable all switches
		CLR	switch1
		CLR	switch2
		CLR	switch3
		CLR	switch4
		CLR	switch5
		CLR	switch6
		CLR	switch7
		CLR	switch8
		CLR	switch9
		CLR	switch10
		CLR	switch11
		CLR	switch12
service_runninglight_loop:
		; running light on switches
		%DELAY
		CLR	switch12	; reset switch 12
		SETB	switch1 	; set switch 1
		%DELAY
		CLR	switch1 	; reset switch 1
		SETB	switch2 	; set switch 2
		%DELAY
		CLR	switch2 	; reset switch 2
		SETB	switch3 	; set switch 3
		%DELAY
		CLR	switch3 	; reset switch 3
		SETB	switch4 	; set switch 4
		%DELAY
		CLR	switch4		; reset switch 4
		SETB	switch5		; set switch 5
		%DELAY
		CLR	switch5		; reset switch 5
		SETB	switch6		; set switch 6
		%DELAY
		CLR	switch6		; reset switch 6
		SETB	switch7		; set switch 7
		%DELAY
		CLR	switch7		; reset switch 7
		SETB	switch8		; set switch 8
		%DELAY
		CLR	switch8		; reset switch 8
		SETB	switch9		; set switch 9
		%DELAY
		CLR	switch9		; reset switch 9
		SETB	switch10	; set switch 10
		%DELAY
		CLR	switch10	; reset switch 10
		SETB	switch11	; set switch 11
		%DELAY
		CLR	switch11	; reset switch 11
		SETB	switch12	; set switch 12
		LJMP	service_runninglight_loop

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

		CSEG	AT  0BH	      		; 0BH is address for Timer 0 interrupt			;!3 cycles this far
		LJMP	timer0int							     		;!2 cycles

?TIMER0		SEGMENT	CODE
		RSEG	?TIMER0

timer0int:	PUSH	PSW	     		; store PSW for safekeeping				;!2 cycles
		PUSH	ACC	      		; store ACC for safekeeping				;!2 cycles

		MOV	C,dmxsignalcheck
		MOV	dmxsignal,C
		CLR	dmxsignalcheck
		JC	blinkpowerled
enablepowerled:	;CLR	_ledpower		; set power LED to indicate that there is no DMX signal going on
		MOV	A,signallostcountdown
		JZ	checkpowerledend
		DEC	signallostcountdown
		SJMP	checkpowerledend
blinkpowerled:	MOV	signallostcountdown,#signallostcountdowninit
		;CPL	_ledpower		; reverse power LED's status for DMX signal indication
checkpowerledend:

timer0int_end:	POP	ACC	      		; restore ACC						;!2 cycles
		POP	PSW	      		; restore PSW						;!2 cycles
		RETI

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

		CSEG	AT  23H	     			; 23H is the address for Serial interrupt		; 3 cycles this far
;		LJMP	serialint											; 2 cycles

;?SERIAL		SEGMENT	CODE
;		RSEG	?SERIAL

serialint:	PUSH	PSW	      			; store PSW for safekeeping				; 2 cycles
		PUSH	ACC	     			; store ACC for safekeeping				; 2 cycle

                JNB	RI,serialint_end     									; 2 cycles

		SETB	dmxsignalcheck										; 1 cycle

                JNB	RB8,dmxbreak										; 2 cycles

		; if counter is zero then skip
		CJNE	dmxpointer,#table,not_zero								; 2 cycles
		SJMP	serialint_end										; 2 cycles
		
dmxbreak: 	; break detected
		MOV	dmxpointer,#table+1						     			; 1 cycle

		MOV	addressh,initaddressh		; retrieve high order address byte			; 2 cycles
		MOV	addressl,initaddressl		; retrieve low order address byte			; 2 cycles

		SJMP	serialint_end								       		; 2 cycles

not_zero:      	; if counter exceeds the table's size then skip
		CJNE	dmxpointer,#table+tablesize,dmxlimitcheck	      					; 2 cycles

dmxlimitcheck:	JNC	serialint_end										; 2 cycles

		CJNE	dmxpointer,#table+1,nostartbyte	; check if this is a start byte being received		; 2 cycles

		; this is a start byte being received
		MOV	A,SBUF				; retrieve start byte					; 1 cycle
		JZ	goodstartbyte                   ; the received start byte is zero			; 2 cycles

badstartbyte:	MOV	dmxpointer,#table		; the received start byte is not zero, reset pointer	; 1 cycle
		SJMP	serialint_end										; 2 cycles

goodstartbyte:	INC	dmxpointer								       		; 1 cycle
		SJMP	serialint_end

nostartbyte:	MOV	A,addressh										; 1 cycle
		JNZ	decrement										; 2 cycles
		MOV	A,addressl										; 1 cycle
		JNZ	decrement										; 2 cycles

		MOV	@dmxpointer,SBUF		; store DMX value					; 2 cycles

		INC	dmxpointer								       		; 1 cycle
		SJMP	serialint_end			; counter is already zero, so don't decrement again	; 2 cycles

decrement:	CLR	C											; 1 cycle
		MOV	A,addressl										; 1 cycle
		SUBB	A,#1											; 1 cycle
		MOV	addressl,A										; 1 cycle
		MOV	A,addressh										; 1 cycle
		SUBB	A,#0	     										; 1 cycle
		MOV	addressh,A								       		; 1 cycle

serialint_end:	CLR	RI				; serial receive data handled		      		; 1 cycle
		POP	ACC				; restore ACC				      		; 2 cycles
		POP	PSW				; restore PSW				      		; 2 cycles
		RETI								       				; 2 cycles

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

		END			; End Of File

